/*
    RC522 I2C Module annonces gare

    www.locoduino.org - fev 2019, vers : v 0.6

  MFRC522.cpp - Library to use ARDUINO RFID MODULE KIT 13.56 MHZ WITH TAGS I2C BY AROZCAN
  arozcan @ https://github.com/arozcan/MFRC522-I2C-Library
  MFRC522.cpp - Based on ARDUINO RFID MODULE KIT 13.56 MHZ WITH TAGS SPI Library BY COOQROBOT
  miguelbalboa @ https://github.com/miguelbalboa/rfid

  Mapping @ http://www.locoduino.org/local/cache-vignettes/L610xH886/rc522_annonces_gare_0-87805.jpg?1552162449
  Si vous utilisez une carte Arduino Mega, SDA est sur le connecteur 20 et SCL sur le connecteur 21.
*/


//#define DEBUG                           // Ne pas oublier de commenter en production

//-------------------------------- Module son -----------------------------------//

#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"       // https://github.com/DFRobot/DFRobotDFPlayerMini
const uint8_t soundVolume = 20;        // Sur 30 volume maxi

SoftwareSerial mySoftwareSerial(10, 11); // RX, TX
DFRobotDFPlayerMini myDFPlayer;

// Plus d'infos sur DFPlayer : https://www.dfrobot.com/wiki/index.php/DFPlayer_Mini_SKU:DFR0299#Connection_Diagram

//-------------------------------- RC522 -----------------------------------//

#include <Wire.h>
#include "MFRC522_I2C.h"              // https://github.com/arozcan/MFRC522-I2C-Library

const uint8_t rstPin = 6;             // Pin for RST
MFRC522 mfrc522(0x28, rstPin);        // MFRC522 instance.

//-------------------------- Variable globales -----------------------------//


const long rc522tempo = 1 * 10UL;  // rc522tempo entre 2 lectures => 1 * 10UL = 10/1000 sec
long prev;
String uidOld;

const uint8_t nbreLocos = 3;          //Remplacer par le nbs de machines. Doit etre < 255

/*
 * Structure permettant de décrire une locomotive 
 */
typedef struct {
  uint8_t index;
  char *  tagRFID; 
  char *  nom;
  char *  identifiant;
} Locomotive;

const Locomotive locosTab[nbreLocos] = {
  {0, "04263722665D81", "YC 6400   ", 0001},
  {1, "04463722665D81", "Renault VH", 0002},
  {2, "04726922665D80", "CC-7126   ", 0003}
};

//-------------------------- Definition des fonctions -----------------------------//

void DFRobotDFPlayerMini_printDetail(uint8_t type, int value);
void mfrc522_printDetail(void);
void DFRobotDFPlayerMini_printDetail(uint8_t type, int value);


//-------------------------------- setup -----------------------------------//

void setup() {
  Serial.begin(115200);                      // Initialize serial communications with the PC

  // SoftwareSerial
  mySoftwareSerial.begin(9600);              // Initialize mySoftwareSerial
  Serial.print(F("\nDFRobot DFPlayer Mini Demo\n"));
  Serial.print(F("Initializing DFPlayer ... (May take 3~5 seconds)\n"));

  if (!myDFPlayer.begin(mySoftwareSerial)) {  // Use softwareSerial to communicate with mp3.
    Serial.print(F("Unable to begin:\n"));
    Serial.print(F("1.Please recheck the connection!\n"));
    Serial.print(F("2.Please insert the SD card!\n"));
  }
  Serial.print(F("DFPlayer Mini online.\n"));
  myDFPlayer.volume(soundVolume);            //Set volume value. From 0 to 30


  Wire.begin();                              // Initialize I2C
  mfrc522.PCD_Init();                        // Initialize MFRC522
  mfrc522_printDetail();                     // Show details of PCD - MFRC522 Card Reader details
  Serial.print(F("\n"));

}

//-------------------------------- loop -----------------------------------//

void loop() {

  mfrc522update(); // MAJ du RC522

  //Print the detail message from DFPlayer to handle different errors and states.
  if (myDFPlayer.available()) {
    DFRobotDFPlayerMini_printDetail(myDFPlayer.readType(), myDFPlayer.read());
  }
}

//-------------------------------- fonctions -----------------------------------//


//////////////////////////////////////////////////////////////////////////////////
//                                mfrc522update
//////////////////////////////////////////////////////////////////////////////////

void mfrc522update() {

  while ( ! mfrc522.PICC_IsNewCardPresent() || ! mfrc522.PICC_ReadCardSerial() ) {
    delay(50);
  }

  if (millis() > prev + rc522tempo ) {            // Verifie que la rc522tempo est depassee
    String uidNew;
    for (uint8_t i = 0; i < mfrc522.uid.size ; i++) {
      uidNew += mfrc522.uid.uidByte[i] < 0x10 ? "0" : "";
      uidNew += String(mfrc522.uid.uidByte[i], HEX);
    }

    uidNew.toUpperCase();            // On met toutes les lettres du tag en majuscules

#ifdef DEBUG
    Serial.println(uidNew);          // Permet de lire n'importe quel tag passe sur le lecteur
                                     // et d'afficher son UID
#endif

    if (uidNew != uidOld) {          // Le tag est il different ?
      for (uint8_t i = 0; i < nbreLocos ; i++) {  // Recherche dans le tableau des locos
        if (uidNew == locosTab[i].tagRFID) {

#ifdef DEBUG
          Serial.print(F("Train : \t"));
          Serial.print(locosTab[i].nom);
          Serial.print(F("\n"));
#endif

          myDFPlayer.playMp3Folder(locosTab[i].identifiant);

          uidOld = uidNew;  // On met a jour la variable uidOld

          break;   // Une loco a ete trouvee, on quitte la boucle de recherche
        }
      }
    }
    prev = millis();       // On affecte a la variable prev l'heure courante
  }
}



//////////////////////////////////////////////////////////////////////////////////
//                                mfrc522_printDetail
//////////////////////////////////////////////////////////////////////////////////

void mfrc522_printDetail() {
  // Get the MFRC522 software version
  byte v = mfrc522.PCD_ReadRegister(mfrc522.VersionReg);
  Serial.print(F("MFRC522 Software Version: 0x"));
  Serial.print(v, HEX);
  if (v == 0x91)
    Serial.print(F(" = v1.0"));
  else if (v == 0x92)
    Serial.print(F(" = v2.0"));
  else
    Serial.print(F(" (unknown)"));
  Serial.println("");
  // When 0x00 or 0xFF is returned, communication probably failed
  if ((v == 0x00) || (v == 0xFF)) {
    Serial.println(F("WARNING: Communication failure, is the MFRC522 properly connected ?"));
  }
}


//////////////////////////////////////////////////////////////////////////////////
//                         DFRobotDFPlayerMini_printDetail
//////////////////////////////////////////////////////////////////////////////////

void DFRobotDFPlayerMini_printDetail(uint8_t type, int value) {
  switch (type) {
    case TimeOut:
      Serial.println(F("Time Out!"));
      break;
    case WrongStack:
      Serial.println(F("Stack Wrong!"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Card Inserted!"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Card Removed!"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Card Online!"));
      break;
    case DFPlayerUSBInserted:
      Serial.println(F("USB Inserted!"));
      break;
    case DFPlayerUSBRemoved:
      Serial.println(F("USB Removed!"));
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Number:"));
      Serial.print(value);
      Serial.println(F(" Play Finished!"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayerError:"));
      switch (value) {
        case Busy:
          Serial.println(F("Card not found"));
          break;
        case Sleeping:
          Serial.println(F("Sleeping"));
          break;
        case SerialWrongStack:
          Serial.println(F("Get Wrong Stack"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Check Sum Not Match"));
          break;
        case FileIndexOut:
          Serial.println(F("File Index Out of Bound"));
          break;
        case FileMismatch:
          Serial.println(F("Cannot Find File"));
          break;
        case Advertise:
          Serial.println(F("In Advertise"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
}

//////////////////////////////////////////////////////////////////////////////////
//                                        End
//////////////////////////////////////////////////////////////////////////////////
